require_relative 'epiphany_trove'
require_relative 'playable'

module StudioGame
  class Philosopher
    include Playable
  
    attr_accessor :health 
    attr_accessor :name
  
    def initialize(name, health=100)
      @name = name.capitalize
      @health = health
      @discovered_epiphanies = Hash.new(0)
    end
  
    def self.from_csv(string)
      name, health = string.split(',')
      Philosopher.new(name, Integer(health))
    end
  
    def name=(new_name)
      @name = new_name.capitalize
    end
  
    def to_s
      "\nI'm #{@name} with health = #{@health}, points = #{points}, and score = #{score}."
    end
  
    def discovered_epiphanies(epiphany)
      @discovered_epiphanies[epiphany.name] += epiphany.points
      puts "#{@name} discovered #{epiphany.name} for #{epiphany.points} points."
      puts "#{@name}'s discoveries: #{@discovered_epiphanies}"
    end
  
    def each_discovered_epiphany
      @discovered_epiphanies.each do |name, points|
        yield Epiphany.new(name, points)
      end
    end
  
    def points
      @discovered_epiphanies.values.reduce(0, :+)
    end
  
    def score
      points + @health 
    end
  
    def <=>(other)
      other.score <=> score
    end
  end

  if __FILE__ == $0
    player = Philosopher.new("camus")
    puts player.name
    puts player.health
    player.w00t
    puts player.health
    player.blam
    puts player.health
  end
end