require_relative 'philosopher'
require_relative 'die'
require_relative 'game_turn'
require_relative 'epiphany_trove'
require_relative 'berserk_player'
require_relative 'sloppy_philosopher'

module StudioGame
  class Game
    attr_reader :title
  
    def initialize(title)
      @title = title.capitalize
      @philosophers = []
    end
  
    def load_players(from_file)
      File.readlines(from_file).each do |line|
        add_player(Philosopher.from_csv(line))
      end
    end
  
    def add_player(player)
      @philosophers << (player)
    end
  
    def play(rounds)
      puts "There are #{@philosophers.size} players in #{@title}:"
     
      @philosophers.each do |player| 
        puts player
      end
    
      epiphanies = EpiphanyTrove::EPIPHANIES
    
      puts "\nThere are #{epiphanies.size} discoveries to be made:"
      epiphanies.each do |epiphany|
        puts "Discovering #{epiphany.name} is worth #{epiphany.points} points"
      end
    
      1.upto(rounds) do |round|
        if block_given?
          break if yield
        end
          puts "\nRound #{round}:"
          @philosophers.each do |player|
            if player.alive?
              GameTurn.take_turn(player)
              case
              when player.berserk?
               2.times {player.w00t}
                puts "#{player.name} has gone berserk!"
                #puts player
              when player.alive?
                puts player
              else puts "\n#{player.name.upcase} has fallen!"
              end
            else puts "\n#{player.name} can no longer acquire points."        
          end
        end
      end
    end
  
    def print_name_and_health(player)
      puts "#{player.name} (#{player.health})"
    end
  
    def print_stats
      puts "\n#{@title} Statistics:"
    
      strong_players, weak_players = @philosophers.partition { |player| player.strong? }
    
      puts "\n#{strong_players.size} strong players:"
      strong_players.each do |player|
        print_name_and_health(player)
      end
      
      puts "\n#{weak_players.size} weak players:"
      weak_players.each do |player|
        print_name_and_health(player)
      end
    
      @philosophers.each do |player|
          puts "\n#{player.name}'s point totals:"
          player.each_discovered_epiphany do |epiphany|
            puts "#{epiphany.points} total points from discovering #{epiphany.name}"
          end
            
          puts "#{player.points} grand total points"
        end
      
        puts "#{total_points} total points from philosophical discoveries"
    
      puts "\n#{@title} High Scores:"
      @philosophers.sort.each do |player|
        puts high_score_entry(player)
      end  
    end
  
    def total_points
      @philosophers.reduce(0) { |sum, player| sum + player.points }
    end
  
    def high_score_entry(player)
      formatted_name = player.name.ljust(20, '.')
      "#{formatted_name} #{player.score}"
    end
  
    def save_high_scores(to_file="high_scores.txt")
      File.open(to_file, "w") do |file|
        file.puts "#{@title} High Scores:"
        @philosophers.sort.each do |player|
          file.puts high_score_entry(player)
        end  
      end
    end
  end

  if __FILE__ == $0
    great_philosophers = Game.new("great_philosophers")
    player1 = Philosopher.new("moe")
    great_philosophers.add_player(player1)
    great_philosophers.play(3)
    great_philosophers.print_stats
  end
end